package com.school.project.controller;

import com.school.project.dto.DepartmentDTO;
import com.school.project.service.implementation.DepartmentServiceImp;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.NoSuchElementException;
import java.util.Optional;

@RestController
public class DepartmentController {
    @Autowired(required=true)
    public DepartmentServiceImp departmentServiceImp;

    @GetMapping("department")
    public ResponseEntity<List<DepartmentDTO>> readAllDepartments() {
        List<DepartmentDTO> departmentDTO = departmentServiceImp.readDepartment();

        if(departmentDTO.isEmpty()){
            return ResponseEntity.status(HttpStatus.NOT_FOUND).build();
        }
        else {
            return ResponseEntity.of(Optional.of(departmentDTO));
        }
    }

    @GetMapping("departmentId/{id}")
    public ResponseEntity<DepartmentDTO> readDepartmentById(@PathVariable("id") Long id){
        try {
                DepartmentDTO departmentDTO=departmentServiceImp.readDepartmentById(id);

                return ResponseEntity.of(Optional.of(departmentDTO));

        }catch (NoSuchElementException e){

            return ResponseEntity.status(HttpStatus.NOT_FOUND).build();
        }

    }

    @GetMapping("departmentName/{name}")
    public ResponseEntity<DepartmentDTO> findDepartmentByName(@PathVariable("name") String name){
        DepartmentDTO departmentDTO=departmentServiceImp.findDepartmentByName(name);

        if(departmentDTO == null){
            return ResponseEntity.status(HttpStatus.NOT_FOUND).build();
        }else {
            return ResponseEntity.of(Optional.of(departmentDTO));
        }
    }

    @PostMapping("departmentSave")
    public ResponseEntity<DepartmentDTO> saveDepartment(@RequestBody DepartmentDTO department){
                DepartmentDTO departmentDTO = departmentServiceImp.saveDepartment(department);

                if(departmentDTO == null){
                    return ResponseEntity.status(HttpStatus.NOT_ACCEPTABLE).build();
                }
                else {
                    return ResponseEntity.status(HttpStatus.OK).body(departmentDTO);
                }
    }

    @PutMapping("department/{id}")
    public ResponseEntity<DepartmentDTO> updateDepartmentById(@RequestBody DepartmentDTO department, @PathVariable("id") Long id){
        DepartmentDTO checkDepartment = departmentServiceImp.readDepartmentById(id);

        if(checkDepartment == null){
            return ResponseEntity.status(HttpStatus.NOT_FOUND).build();
        }
        else {
            DepartmentDTO departmentDTO = departmentServiceImp.updateDepartment(department,id);

            return ResponseEntity.of(Optional.of(departmentDTO));
        }
    }

    @DeleteMapping("department/{id}")
    public ResponseEntity<Object> deleteDepartmentById(@PathVariable("id") Long id){
        departmentServiceImp.deleteDepartmentById(id);

        DepartmentDTO departmentDTO = departmentServiceImp.readDepartmentById(id);

        if(departmentDTO == null){
            return ResponseEntity.status(HttpStatus.OK).build();
        }else {

            return  ResponseEntity.of(Optional.of("Department Already Not Exist"));
        }

    }
}
