package com.school.project.service.implementation;

import com.school.project.model.User;
import com.school.project.repository.UserRepository;
import com.school.project.service.interfaces.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import java.util.HashSet;
import java.util.List;
import java.util.Optional;
import java.util.Set;


@Service
public class UserServiceImpl implements UserService, UserDetailsService {

    @Autowired
    private UserRepository userRepository;

    @Autowired
    private PasswordEncoder passwordEncoder;

    @Override
    public Integer saveUser(User user) {
        String password = user.getPassword();
        String encodedPassword = passwordEncoder.encode(password);

        user.setPassword(encodedPassword);

        user = userRepository.save(user);

        return Math.toIntExact(user.getId());
    }

    @Override
    public UserDetails loadUserByUsername(String email) throws UsernameNotFoundException {

        Optional<User> userEmail = userRepository.findUserByEmail(email);

        org.springframework.security.core.userdetails.User springUser=null;

        if(userEmail.isEmpty()){
            throw  new UsernameNotFoundException("User With Email=" + email + "Not Founnd");
        }else{

            User user = userEmail.get();

            List<String> userRoles = user.getRoles();

            Set<GrantedAuthority> userAuthorities = new HashSet<>();

            for (String userRole:userRoles){
                userAuthorities.add(new SimpleGrantedAuthority(userRole));
            }

            springUser = new org.springframework.security.core.userdetails.User(
                    email,
                    user.getPassword(),
                    userAuthorities );
        }

        return springUser;
    }
}
